var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
var localTree = {
	data 		: new Array,
    rowCount	: 0,
	localSize	: 0,
	localAvailableDiskSpace : 0,
    getCellText : function(row, column) {														// text for the files
		if (row == -1 || !this.data[row].exists())
			return " ";
		switch(column) {
			case "localname":
				return this.data[row].leafName;
			case "localsize":
				if (this.data[row].isDirectory())
					return "";
				if (this.data[row].fileSize == 0)
					return "0 KB  ";
				return commas(Math.floor(this.data[row].fileSize / 1024) + 1) + " KB  ";
			case "localdate":
				var date = new Date(this.data[row].lastModifiedTime);
				if ((new Date()).getYear() < date.getYear())									// if not current year, display old year
					return months[date.getMonth()] + ' ' + date.getDate() + ' ' + date.getYear();
				else {
					var time =  date.toLocaleTimeString();										// else display time
					return months[date.getMonth()] + ' ' + date.getDate() + ' ' + time.substring(0, time.lastIndexOf(':'));
				}
			case "localattr":
				if (this.data[row].permissions)
					return this.data[row].permissions;
				return "";
			default:
				return " "; // shouldn't get here
		}
    },
	getImageSrc	: function(row,col)	{															// icons for the files
		if (row == -1)
			return "";
		if (col == "localname") {
			if (!this.data[row].exists())
				return;
			if (this.data[row].isDirectory())
				return "res/directory.ico";
			var fileURI = ios.newFileURI(this.data[row]);
			return "moz-icon://" + fileURI.spec + "?size=16";	// thanks to alex sirota!
		}
	},
	cycleHeader : function(col, elem) {
		var sortDirection = elem.getAttribute("sortDirection") == "ascending" || elem.getAttribute("sortDirection") == "natural"
							? "descending" : "ascending";
		document.getElementById('localname').setAttribute("sortDirection", "natural");
		document.getElementById('localsize').setAttribute("sortDirection", "natural");
		document.getElementById('localdate').setAttribute("sortDirection", "natural");
		document.getElementById('localattr').setAttribute("sortDirection", "natural");
		elem.setAttribute("sortDirection", sortDirection);
		this.sort();
	},
	getCellProperties	: function(row, col, props)		{},
    getColumnProperties	: function(colid, col, props)	{},
	getLevel			: function(row)					{ return 0; },
	getRowProperties	: function(row, props)			{},
	isContainer			: function(row)					{ return false; },
	isSeparator			: function(row)					{ return false; },
	isSorted			: function(row)					{ return false; },
	setTree				: function(treebox)				{ this.treebox = treebox; },

   	// ************************************************* mouseEvent *****************************************************

	dblClick : function(event) {
		if (event.button != 0 || event.originalTarget.localName != "treechildren" || this.selection.count == 0)
			return;

		if (this.data[this.selection.currentIndex].isDirectory())								// if it's a directory
			localDirTree.changeDir(this.data[this.selection.currentIndex].path);				// navigate to it
		else
			transfer(false);																	// else upload the file
	},
	click : function(event) {
		if (event.button == 1 && !document.getElementById('localPasteContext').disabled)		// middle-click paste
			this.paste();
	},
	mouseOver : function(event) {																// display local folder info
		if (this.rowCount)
			document.getElementById('statustxt').label =
				strbundle.getString("localListing") + " " + this.rowCount + " " + strbundle.getString("objects") + " "
			  + commas(this.localSize) + ", " + strbundle.getString("diskSpace") + " " + this.localAvailableDiskSpace;
		else
			document.getElementById('statustxt').label = strbundle.getString("localListingNoObjects");
	},

	// ************************************************* keyEvent *****************************************************

	keyPress : function(event) {
		if (this.rowCount == 0 || this.selection.count == 0)
			return;
		if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
			this.selection.currentIndex = this.rowCount - 1;

		if (event.keyCode == 13) {								// enter
			if (this.selection.count == 1 && this.data[this.selection.currentIndex].isDirectory())		// if it's a directory
				localDirTree.changeDir(this.data[this.selection.currentIndex].path);					// navigate to it
			else
				transfer(false);																		// else upload a file
		} else if (event.ctrlKey && (event.which == 65 || event.which == 97)) {
			event.preventDefault();								// ctrl-a: select all
			this.selection.selectAll();
		} else if (event.ctrlKey && event.which == 32)			// ctrl-space, select or deselect
			this.selection.toggleSelect(this.selection.currentIndex);
		else if (event.keyCode == 8) {							// backspace
			event.preventDefault();
			localDirTree.cdup();
		}
		else if (event.keyCode == 116) {						// F5
			event.preventDefault();
			this.refresh();
		} else if (event.keyCode == 113)						// F2
			this.rename();
		else if (event.charCode == 109 && event.ctrlKey) {		// ctrl-m
			event.preventDefault();
			this.createDirectory();
		} else if (event.keyCode == 46)							// del
			this.remove();
		else if (event.keyCode == 93) {							// display context menu
			var x = {};		var y = {};		var width = {};		var height = {};
			this.treebox.getCoordsForCellItem(this.selection.currentIndex, "localname", "text", x, y, width, height);
			document.getElementById('localmenu').showPopup(
				gLocalTreeChildren, gLocalTreeChildren.boxObject.x + 75, gLocalTreeChildren.boxObject.y + y.value + 5, "context");
		} else if (event.charCode == 112 && event.ctrlKey) {	// ctrl-p
			event.preventDefault();
			this.showProperties();
		} else if (event.charCode == 120 && event.ctrlKey) {	// ctrl-x
			event.preventDefault();
			this.cut();
		} else if (event.charCode == 99 && event.ctrlKey) {		// ctrl-c
			event.preventDefault();
			this.copy();
		} else if (event.charCode == 118 && event.ctrlKey) {	// ctrl-v
			event.preventDefault();
			this.paste();
		}
	},

	// ****************************************************** file functions ***************************************************

	constructPath : function(parent, leafName) {
		return parent + (parent.charAt(parent.length - 1) != slash ? slash : '') + leafName;
	},
	launch : function() {
		if (this.rowCount > 0 && this.selection.count > 0) {
			if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
				this.selection.currentIndex = this.rowCount - 1;
			localFile.launch(this.data[this.selection.currentIndex]);
		}
	},
	createDirectory : function() {
		if (localFile.createDirectory())
			localTree.refresh();
	},
	remove : function() {
		if (this.selection.count == 0)
			return;

		var prompt = true;
		for (var x = 0; x < this.rowCount; ++x)
			if (this.selection.isSelected(x)) {
				if (!localFile.remove(this.data[x], prompt, this.selection.count))
					break;
				prompt = false;
			}
		
		localTree.refresh();
	},
	rename : function() {
		if (this.rowCount > 0 && this.selection.count > 0) {
			if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
				this.selection.currentIndex = this.rowCount - 1;
			if (localFile.rename(this.data[this.selection.currentIndex]))
				localTree.refresh();
		}
	},
	showProperties : function() {
		if (this.rowCount > 0 && this.selection.count > 0) {
			if (this.selection.currentIndex < 0 || this.selection.currentIndex >= this.rowCount)
				this.selection.currentIndex = this.rowCount - 1;
			localFile.showProperties(this.data[this.selection.currentIndex]);
		}
	},

	// ****************************************************** refresh ***************************************************

	refresh : function(skipLocalTree) {
		if (localDirTree.data[localDirTree.selection.currentIndex].open) {							// if localDirTree is open
			localDirTree.toggleOpenState(localDirTree.selection.currentIndex);						// close it up
			localDirTree.data[localDirTree.selection.currentIndex].children = null;					// reset its children
			localDirTree.toggleOpenState(localDirTree.selection.currentIndex);						// and open it up again
		} else
			localDirTree.data[localDirTree.selection.currentIndex].empty = false;					// not empty anymore
		if (!skipLocalTree)
			setTimeout("localTree.updateView()", 1000);												// update localTree, after a little bit
	},

	// ****************************************************** updateView ***************************************************

	updateView : function(column, direction) {
		var localTreeItems = new Array;

		try {
			this.localSize = 0;
			var dir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
			dir.initWithPath(gLocalPath.value);
			this.localAvailableDiskSpace = parseSize(dir.diskSpaceAvailable);						// get local disk size
			var entries = dir.directoryEntries;
			while (entries.hasMoreElements()) {
				var file = entries.getNext().QueryInterface(Components.interfaces.nsILocalFile);
				if (file.exists() && (!file.isHidden() || hiddenmode)) {
					localTreeItems.push(file);
					this.localSize += file.fileSize;
				}
			}
		} catch (ex) {
			error(strbundle.getString("noPermission"));
			debug(ex,31);
			return;
		}
		this.localSize = parseSize(this.localSize);													// get directory size

		this.data = localTreeItems;																	// update localTree
		this.sort();

		localDirTree.reselectCurrentDirectory();													// select directory in localDirTree
		if (this.data.length)
			this.selection.select(0);																// select first element in localTree
	},
	sort : function() {
		if (document.getElementById('localname').getAttribute("sortDirection") &&
				document.getElementById('localname').getAttribute("sortDirection") != "natural") {
			this.data.sort(compareLocalName);
			if (document.getElementById('localname').getAttribute("sortDirection") == "ascending")
				this.data.reverse();
		}
		if (document.getElementById('localsize').getAttribute("sortDirection") && 
				document.getElementById('localsize').getAttribute("sortDirection") != "natural") {
			this.data.sort(compareLocalSize);
			if (document.getElementById('localsize').getAttribute("sortDirection") == "ascending")
				this.data.reverse();
		}
		if (document.getElementById('localdate').getAttribute("sortDirection") &&
				document.getElementById('localdate').getAttribute("sortDirection") != "natural") {
			this.data.sort(compareLocalDate);
			if (document.getElementById('localdate').getAttribute("sortDirection") == "ascending")
				this.data.reverse();
		}
		if (document.getElementById('localattr').getAttribute("sortDirection") &&
				document.getElementById('localattr').getAttribute("sortDirection") != "natural") {
			this.data.sort(compareLocalAttr);
			if (document.getElementById('localattr').getAttribute("sortDirection") == "ascending")
				this.data.reverse();
		}

		this.treebox.rowCountChanged(0, -this.rowCount);
		this.rowCount = this.data.length;
		this.treebox.rowCountChanged(0, this.rowCount);
	},

	// ************************************************* cut, copy, paste *****************************************************

	isCut : false,
	pasteFiles : new Array,
	oldParent : "",
	cut : function() {
		this.copy(true);
	},
	copy : function(isCut) {
		if (this.selection.count == 0)
			return;

		this.isCut = isCut;
		this.pasteFiles = new Array;
		this.oldParent = gLocalPath.value;
		for (var x = 0; x < this.rowCount; ++x)											// put files to be cut/copied in an array to be used when pasting
			if (this.selection.isSelected(x))
				this.pasteFiles.push(this.data[x]);

		document.getElementById('localPasteContext').setAttribute("disabled", false);	// enable pasting
	},
	paste : function() {
		if (this.pasteFiles.count == 0)
			return;

		var newParent = gLocalPath.value;
		if (newParent.indexOf(this.pasteFiles[0].path) != -1)							// can't copy into a subdirectory of itself
			return;
		
		var prompt = true;
		var skipall = false;
		var overwrite = false;

		try {
			var newDir = localDirTree.getFile(localDirTree.indexOfPath(newParent));
			for (var x = 0; x < this.pasteFiles.length; ++x) {
				var newFile = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
				newFile.initWithPath(this.constructPath(newDir.path, this.pasteFiles[x].leafName));

				if (newFile.exists() && skipall)
					continue;
				if (newFile.exists() && prompt) {										// ask nicely
					var response = new Object();
					response.value = 0;
					var timer = destructmode;

					window.openDialog("chrome://fireftp/content/confirmFileDialog.xul","confirmFileDialog",
						"screenY="+(document.getElementById('main-window').boxObject.height/2 - 45+document.getElementById('main-window').boxObject.screenY)
					  +",screenX="+(document.getElementById('main-window').boxObject.width/2 - 200+document.getElementById('main-window').boxObject.screenX)+
						",width=400,height=90,chrome,modal=yes,dialog=yes,resizable=yes", response, this.pasteFiles[x].leafName, true, true, timer);
					if (response.value == 1)
						overwrite = true;
					if (response.value == 2)
						prompt = false;
					else if (response.value == 3)
						continue;
					else if (response.value == 4 || response.value == 0)
						return;
					else if (response.value == 5) {
						skipall = true;
						continue;
					}
				}

				if (newFile.exists() && (!prompt || overwrite)) {
					newFile.remove(true);
					overwrite = false;
				}

				if (this.isCut)
					this.pasteFiles[x].moveTo(newDir, null);							// cut
				else
					this.pasteFiles[x].copyTo(newDir, null);							// or copy
			}
		} catch (ex) {
			debug(ex,36);
			error(strbundle.getString("pasteError"));
		}
		
		if (this.isCut) {																// reset variables if it was a cut
			localDirTree.selection.currentIndex = localDirTree.indexOfPath(this.oldParent);
			this.refresh(true);
			localDirTree.selection.currentIndex = localDirTree.indexOfPath(newParent);
			this.pasteFiles = new Array;
			this.isCut = false;
			document.getElementById('localPasteContext').setAttribute("disabled", true);
		}

		this.refresh();
	},

	// ************************************************* dragEvent *****************************************************

	dragOver : function(event) {
		var dragSession = whataDrag.getCurrentSession();
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);

		trans.addDataFlavor("text/unicode");
		dragSession.getData(trans, 0);

		var bestFlavor = new Object();		var dataObj = new Object();		var len = new Object();
		trans.getAnyTransferData(bestFlavor, dataObj, len);
		dataObj = dataObj.value.QueryInterface(Components.interfaces.nsISupportsString);

		var row = {};		var col = {};		var child = {};
		this.treebox.getCellAt(event.pageX, event.pageY, row, col, child);

		dragSession.canDrop = dataObj.data == "remotetreechildren";
	},
	dragDrop : function(event) {
		var dragSession = whataDrag.getCurrentSession();
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
	
		trans.addDataFlavor("text/unicode");
		dragSession.getData(trans, 0);

		var bestFlavor = new Object();		var dataObj = new Object();		var len = new Object();
		trans.getAnyTransferData(bestFlavor, dataObj, len);
		dataObj = dataObj.value.QueryInterface(Components.interfaces.nsISupportsString);

		if (dataObj.data == "remotetreechildren")												// download file
			transfer(true);
		event.preventBubble();
	},
};
